﻿Imports System.IO
Imports System.Windows.Media.Imaging

Partial Public Class MainPage
    Inherits UserControl

    '保存ボタンをクリックしたときの処理です
    Private Sub savePaint_Click(ByVal sender As System.Object, ByVal e As System.Windows.RoutedEventArgs)
        'SaveFileDialogを開きます
        Dim saveDialog As SaveFileDialog = New SaveFileDialog() With { _
            .DefaultExt = "bmp", _
            .Filter = "ビットマップ(*.bmp)|*.bmp" _
            }

        If saveDialog.ShowDialog() = True Then
            'ファイルの保存するためのストリームを開きます
            Using stream As Stream = saveDialog.OpenFile()

                '画像を取り込んだInkPresenterを対象にWriteableBitmapを生成します
                Dim wb As WriteableBitmap = New WriteableBitmap(Me.inkP, Nothing)


                'WriteableBitmapから出力用のバイト配列を取得します
                Dim buffer As Byte() = GetBuffer(wb)

                'Bitmapファイルとして保存します
                stream.Write(buffer, 0, buffer.Length)
            End Using
        End If
    End Sub

   
    'WriteableBitmapから出力用のbyte配列を取得します
    Private Function GetBuffer(ByVal bitmap As WriteableBitmap) As Byte()
        Dim width As Integer = bitmap.PixelWidth
        Dim height As Integer = bitmap.PixelHeight

        Dim ms As MemoryStream = New MemoryStream()

        'BMP File Header(14 bytes)
        'the magic number(2 bytes):BM
        ms.WriteByte(&H42)
        ms.WriteByte(&H4D)

        'the size of the BMP file in bytes(4 bytes)
        Dim len As Integer = bitmap.Pixels.Length * 4 + &H36
        Dim mask As Integer = &HFF

        ms.WriteByte(CByte(len And mask))
        ms.WriteByte(CByte((len >> 8) And mask))
        ms.WriteByte(CByte((len >> 16) And mask))
        ms.WriteByte(CByte((len >> 24) And mask))

        'reserved(2 bytes)
        ms.WriteByte(&H0)
        ms.WriteByte(&H0)

        'reserved(2 bytes)
        ms.WriteByte(&H0)
        ms.WriteByte(&H0)

        'the offset(4 bytes)
        ms.WriteByte(&H36)
        ms.WriteByte(&H0)
        ms.WriteByte(&H0)
        ms.WriteByte(&H0)

        'Bitmap Information(40 bytes:Windows V3)
        'the size of this header(4 bytes)
        ms.WriteByte(&H28)
        ms.WriteByte(&H0)
        ms.WriteByte(&H0)
        ms.WriteByte(&H0)

        'the bitmap width in pixels(4 bytes)
        ms.WriteByte(CByte(width And mask))
        ms.WriteByte(CByte((width >> 8) And mask))
        ms.WriteByte(CByte((width >> 16) And mask))
        ms.WriteByte(CByte((width >> 24) And mask))

        'the bitmap height in pixels(4 bytes)
        ms.WriteByte(CByte(height And mask))
        ms.WriteByte(CByte((height >> 8) And mask))
        ms.WriteByte(CByte((height >> 16) And mask))
        ms.WriteByte(CByte((height >> 24) And mask))

        'the number of color planes(2 bytes)
        ms.WriteByte(&H1)
        ms.WriteByte(&H0)

        'the number of bits per pixel(2 bytes)
        ms.WriteByte(&H20)
        ms.WriteByte(&H0)

        'the compression method(4 bytes)
        ms.WriteByte(&H0)
        ms.WriteByte(&H0)
        ms.WriteByte(&H0)
        ms.WriteByte(&H0)

        'the image size(4 bytes)
        ms.WriteByte(&H0)
        ms.WriteByte(&H0)
        ms.WriteByte(&H0)
        ms.WriteByte(&H0)

        'the horizontal resolution of the image(4 bytes)
        ms.WriteByte(&H0)
        ms.WriteByte(&H0)
        ms.WriteByte(&H0)
        ms.WriteByte(&H0)

        'the vertical resolution of the image(4 bytes)
        ms.WriteByte(&H0)
        ms.WriteByte(&H0)
        ms.WriteByte(&H0)
        ms.WriteByte(&H0)

        'the number of colors in the color palette(4 bytes)
        ms.WriteByte(&H0)
        ms.WriteByte(&H0)
        ms.WriteByte(&H0)
        ms.WriteByte(&H0)

        'the number of important colors(4 bytes)
        ms.WriteByte(&H0)
        ms.WriteByte(&H0)
        ms.WriteByte(&H0)
        ms.WriteByte(&H0)

        'Bitmap data
        For y As Integer = height - 1 To 0 Step -1
            For x As Integer = 0 To width - 1
                Dim pixel As Integer = bitmap.Pixels(width * y + x)
                ms.WriteByte(CByte(pixel And &HFF)) 'B
                ms.WriteByte(CByte((pixel >> 8) And &HFF)) 'G
                ms.WriteByte(CByte((pixel >> &H10) And &HFF)) 'R
                ms.WriteByte(&H0) 'reserved
            Next x
        Next y

        Return ms.GetBuffer()
    End Function

    Public Sub New()
        InitializeComponent()
    End Sub

    '現在処理中のStrokeオブジェクトを保持します
    Private currentStroke As System.Windows.Ink.Stroke

    'マウスが左クリックされたとき、InkPresenterに新しいStrokeを追加します
    Private Sub inkP_MouseLeftButtonDown(ByVal sender As System.Object, ByVal e As System.Windows.Input.MouseButtonEventArgs)
        Me.inkP.CaptureMouse()
        currentStroke = New System.Windows.Ink.Stroke()
        currentStroke.StylusPoints.Add(e.StylusDevice.GetStylusPoints(inkP))
        currentStroke.DrawingAttributes.Color = Colors.Black
        currentStroke.DrawingAttributes.Width = 5
        currentStroke.DrawingAttributes.Height = 5
        inkP.Strokes.Add(currentStroke)
    End Sub

    'マウスが移動されたとき、Strokeに新しいStylusPointを追加します
    Private Sub inkP_MouseMove(ByVal sender As System.Object, ByVal e As System.Windows.Input.MouseEventArgs)
        If currentStroke Is Nothing Then
            Return
        End If

        currentStroke.StylusPoints.Add(e.StylusDevice.GetStylusPoints(inkP))
    End Sub

    'マウスのキャプチャを失ったとき、現在のStrokeに対する処理を終了します
    Private Sub inkP_LostMouseCapture(ByVal sender As System.Object, ByVal e As System.Windows.Input.MouseEventArgs)
        currentStroke = Nothing
    End Sub

    '画像を選択ボタンをクリックしたときの処理です
    Private Sub imageSelect_Click(ByVal sender As System.Object, ByVal e As System.Windows.RoutedEventArgs)
        Me.selectedImage.Text = Nothing
        Dim dialog As OpenFileDialog = New OpenFileDialog()
        dialog.Multiselect = False '単一選択    
        Dim dialogResult As Boolean? = dialog.ShowDialog()

        If Not dialogResult.HasValue Or Not dialogResult.Value Then
            Return 'ファイルが選択されなかったときは何もしない    
        End If

        Try
            '選択した画像ファイルを読み込み、       
            Dim source As BitmapImage = New BitmapImage()
            source.SetSource(dialog.File.OpenRead())
            'Imageのソースに設定        
            Me.image.Source = source
            Me.selectedImage.Text = dialog.File.Name
        Catch
            MessageBox.Show("画像が正しく読み込めませんでした")
        End Try
    End Sub

End Class